/*
 *   Copyright 2010 Brian Rosenberger (Brutex Network)
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */
package net.brutex.xservices.types.ant;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * Generic key/ value pairs.
 *
 * @author Brian Rosenberger
 */
@XmlRootElement
public class AntProperty {

    /**
     * Key of the entry.
     */
    @XmlElement(required=true)
    public String name ="";

    /**
     * Value of the entry.
     */
    @XmlElement(required=true)
    public String value="";

    /**
     * Converts a Map&lt;String, String&gt; into a list of
     * AntProperties.
     * @param newMap   The map to convert
     * @return      A list of key/value pairs
     */
    public static List<AntProperty> createAntPropertyList(Map<String, Object> newMap) {
        List<AntProperty> list = new ArrayList<AntProperty>();
        for(Map.Entry<String, Object> e : newMap.entrySet()) {
            list.add(new AntProperty(e.getKey(), (String)e.getValue()));
        }
        return list;
    }

    /**
     * Creates a new AntProperty.
     * @param name
     * @param value
     */
    public AntProperty(String name, String value) {
        this.name = name;
        this.value = value;
    }

    /**
     * Creates a new AntProperty.
     */
    public AntProperty() {
    }
    

}
