/*
 *   Copyright 2010 Brian Rosenberger (Brutex Network)
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

package net.brutex.xservices.ws;

import java.util.GregorianCalendar;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.ws.WebFault;

import net.brutex.xservices.util.BrutexNamespaces;

/**
 * Generic web service fault.
 *
 * @author Brian Rosenberger, bru@brutex.de
 * since 0.4.0
 */
@WebFault(targetNamespace=BrutexNamespaces.WS_XSERVICES)
public class XServicesFault extends Exception {

    /**
	 * 
	 */
	private static final long serialVersionUID = -6779279189376374820L;

	public XServicesFault(String message, Exception e) {
        this(message, e.getCause());
    }

    public XServicesFault(String string) {
        this(string, new Exception(string).getCause());
    }

    public XServicesFault(Exception e) {
        this(e.getMessage(), e.getCause());
    }

    public XServicesFault(String message, Throwable cause) {
        super(message, cause);
        this.faultstring=message;

       
        try {
            timestamp = DatatypeFactory.newInstance().newXMLGregorianCalendar(new GregorianCalendar());
        } catch (DatatypeConfigurationException ex) {
            System.err.println(ex.getMessage());
        }


    }
    /**
     * The error message.
     */
    @XmlElement(name="faultstring", namespace=BrutexNamespaces.WS_XSERVICES)
    public String faultstring = "";

    /**
     * Username under which the web service has been executed.
     */
    @XmlElement(name="username", namespace=BrutexNamespaces.WS_XSERVICES)
    public String username = System.getProperty("user.name");

    /**
     * Home directory of the user profile running the web service.
     */
    @XmlElement(name="homedir", namespace=BrutexNamespaces.WS_XSERVICES)
    public String homedir = System.getProperty("user.home");

    
    /**
     * Timestamp when the fault was thrown.
     */
    @XmlElement(name="timstamp", namespace=BrutexNamespaces.WS_XSERVICES)
    public XMLGregorianCalendar timestamp = null;
    
    /**
     * Java runtime version.
     */
    @XmlElement(name="jvmversion")
    public String jvmruntime = System.getProperty("java.version");
}
